@@ -61,8 +61,12 @@ get_deb (DVContents *deb)
 		GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
 	current_path = dv_get_working_path (deb);
 	if (current_path)
-		gtk_file_chooser_set_current_folder 
-			(GTK_FILE_CHOOSER (chooser), current_path);
+	{
+		gchar * tmp;
+		tmp = g_path_get_dirname(current_path);
+		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser), tmp);
+		g_free (tmp);
+	}
 	else
 		gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
 			"/var/cache/apt/archives");
@@ -85,12 +89,14 @@ get_deb (DVContents *deb)
 		filename = gtk_file_chooser_get_filename
 			(GTK_FILE_CHOOSER (chooser));
 	gtk_widget_destroy (chooser);
+	dv_set_working_path (deb, filename);
 	return filename;
 }
 
 void
 on_open_deb (GtkMenuItem * menuitem, gpointer user_data)
 {
+	GFile * gfile;
 	gchar *filename;
 	DVContents *deb;
 
@@ -100,7 +106,9 @@ on_open_deb (GtkMenuItem * menuitem, gpointer user_data)
 	filename = get_deb (deb);
 	if (filename)
 	{
-		open_deb (filename, deb);
+		gfile = g_file_new_for_path (filename);
+		set_deb_file (gfile, deb);
+		open_deb (deb);
 		g_free (filename);
 	}
 }
@@ -234,23 +242,28 @@ on_dvtoolopen_clicked (GtkToolButton *toolbutton, gpointer user_data)
 	GtkWidget * parent;
 	gchar *filename;
 	DVContents *deb;
+	GFile * gfile;
 
 	deb = (DVContents*)lookup_widget 
 		(GTK_WIDGET(toolbutton), "context");
 	g_return_if_fail(deb);
-	filename = NULL;
+	filename = dv_get_working_path (deb);
 	filename = get_deb (deb);
 	if (!filename)
 		return;
+	gfile = g_file_new_for_path (filename);
+	if (!g_file_query_exists (gfile, NULL))
+		return;
 	if (g_str_has_suffix(filename, ".changes"))
-		dv_parse_changes(filename);
+		dv_parse_changes(gfile, deb);
 	else
 	{
 		parent = dv_get_parent(deb);
 		dv_archive_clear (deb);
 		g_object_set_data (G_OBJECT(parent), "context", deb);
 		set_dv_parent (parent);
-		open_deb (filename, deb);
+		set_deb_file (gfile, deb);
+		open_deb (deb);
 		g_free (filename);
 	}
 }
@@ -266,17 +279,23 @@ new_window_init (GtkWidget *widget)
 {
 	GtkWidget * window1;
 	gchar * filename;
-	DVContents *deb;
+	DVContents * original, *deb;
+	GFile * gfile;
 
-	deb = (DVContents*) lookup_widget (widget, DVCONTENTS);
-	filename = dv_get_working_path (deb);
+	original = (DVContents*) lookup_widget (widget, DVCONTENTS);
+	/* get the working path from the current context */
+	filename = dv_get_working_path (original);
 	deb = dv_create ();
+	/* insert it into the new context */
 	dv_set_working_path (deb, filename);
 	filename = get_deb(deb);
 	if (!filename)
 		return;
+	gfile = g_file_new_for_path (filename);
+	if (!g_file_query_exists (gfile, NULL))
+		return;
 	if (g_str_has_suffix(filename, ".changes"))
-		dv_parse_changes(filename);
+		dv_parse_changes(gfile, deb);
 	else
 	{
 		window1 = create_deb_gview_window ();
@@ -285,8 +304,8 @@ new_window_init (GtkWidget *widget)
 		set_dv_parent (GTK_WIDGET(window1));
 		gtk_widget_show (window1);
 		dv_add_child (GTK_WINDOW(window1));
-		set_deb_file (filename, deb);
-		open_deb (filename, deb);
+		set_deb_file (gfile, deb);
+		open_deb (deb);
 		g_free(filename);
 	}
 }
@@ -345,7 +364,7 @@ on_treeview_button_press_event (GtkWidget *widget,
 			pop = create_dvpopmenu ();
 			gtk_menu_attach_to_widget (GTK_MENU (pop), widget, NULL);
 			gtk_menu_popup (GTK_MENU (pop), NULL, NULL, NULL, NULL, 
-                  event->button, gtk_get_current_event_time ());
+				event->button, gtk_get_current_event_time ());
 			break;
 		}
 	}
