@@ -2,7 +2,7 @@
  *            dvarchive.c
  *
  *  Tue Aug  1 00:50:52 2006
- *  Copyright  2006, 2007  Neil Williams
+ *  Copyright  2006-2008  Neil Williams
  *  linux@codehelp.co.uk
  * Copyright (C) 1994,1995 Ian Jackson <ian@chiark.greenend.org.uk>
  *******************************************************************/
@@ -31,7 +31,7 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <glib/gstdio.h>
-#include <libgnomevfs/gnome-vfs.h>
+#include <gio/gio.h>
 #include <gtk/gtk.h>
 #include <ar.h>
 #include <archive.h>
@@ -58,32 +58,28 @@ typedef enum
 	DV_ERR_COMPRESS,
 } DVError;
 
+/* details of the files within the .deb */
 typedef struct
 {
 	gint64 size;
 	gpointer content;
 	gboolean compressed;
-	gpointer gzbuffer;
 	GtkWidget *parent;
-	const gchar * filewithin;
-	gboolean image; /*unused so far */
 }DVFileData;
 
 struct DV_s
 {
 	const gchar *filename;
 	gpointer readbuffer;
-	gchar *pkg_name;
-	gchar *author_name;
 	gchar *copyright;
-	gint64 expand_size;
 	GHashTable *control_files;
 	GHashTable *data_files;
 	struct archive *control_tar;
 	struct archive *data_tar;
 	struct ar_hdr *current;
 	gulong skip;
-	GnomeVFSHandle * vfshandle;
+	GFile * ghandle;
+	GInputStream * stream;
 	GtkWidget *parent;
 	DVError error;
 	gboolean store_set;
@@ -91,6 +87,8 @@ struct DV_s
 	const gchar * ar_message; /* error string from archive/ar_hdr */
 	gchar * selected_file;
 	gchar * current_path;
+	/** indicates if the GFile is a URI or a path */
+	gboolean needs_uri;
 };
 
 enum compression_type
@@ -108,57 +106,96 @@ dv_create (void)
 	return deb;
 }
 
+void
+dv_set_needs_uri (DVContents * deb, gboolean val)
+{
+	g_return_if_fail (deb);
+	deb->needs_uri = val;
+}
+
+gboolean
+dv_get_needs_uri (DVContents * deb)
+{
+	/* default value is FALSE and the toolbar callbacks expect FALSE */
+	return deb->needs_uri;
+}
+
 static void
-dv_new_window (const gchar * debname)
+dv_new_window (GFile * deb)
 {
 	DVContents * context;
 	GtkWindow *window1;
+	gchar * filename;
 
-	g_return_if_fail (debname);
+	g_return_if_fail (deb);
 	context = dv_create();
 	window1 = GTK_WINDOW(create_deb_gview_window ());
 	g_return_if_fail (window1);
 	g_object_set_data (G_OBJECT(window1), DVCONTENTS, context);
 	set_dv_parent (GTK_WIDGET(window1));
-	set_deb_file (debname, context);
+	context->ghandle = deb;
+	filename = g_file_get_path (deb);
+	/* set the working path for the new window from the .changes file */
+	dv_set_working_path (context, filename);
+	set_deb_file (deb, context);
 	gtk_widget_show (GTK_WIDGET(window1));
 	dv_add_child (window1);
 }
 
 void
-dv_parse_changes (const gchar * cfilename)
+dv_parse_changes (GFile * cfile, DVContents * deb)
 {
 	gchar **changes_file;
-	gchar * debname, *debline, *contents, *chgpath;
+	gchar * debname, *debline, *contents, *chgpath, *path;
 	guint c;
-	gchar *vfstext;
-	GnomeVFSURI * vfsuri;
-	GnomeVFSResult vfsresult;
-	gint changes_size;
+	GFile * parent;
+	GError * result;
+	gsize changes_size;
+	gboolean files;
 
-	g_return_if_fail (cfilename);
-	vfstext = gnome_vfs_make_uri_from_input (cfilename);
-	vfsuri = gnome_vfs_uri_new (vfstext);
-	chgpath = gnome_vfs_uri_to_string
-		(gnome_vfs_uri_resolve_relative(vfsuri, "./"), GNOME_VFS_URI_HIDE_NONE);
-	vfsresult = gnome_vfs_read_entire_file (vfstext, &changes_size, &contents);
-	gnome_vfs_uri_unref (vfsuri);
+	g_return_if_fail (cfile);
+	result = NULL;
+	files = FALSE;
+	g_file_load_contents (cfile, NULL, &contents, &changes_size, 
+		NULL, &result);
+	if (result)
+	{
+		g_warning ("Unable to parse .changes file: %s", result->message);
+		g_clear_error (&result);
+		return;
+	}
+	path = g_file_get_parse_name (cfile);
+	chgpath = NULL;
+	parent = g_file_get_parent (cfile);
+	chgpath = g_file_get_uri (parent);
 	changes_file = g_strsplit (contents, "\n", -1);
 	for (c = 0; c < g_strv_length(changes_file); c++)
 	{
+		// handle new format .changes files
+		if (g_str_has_prefix (changes_file[c], "Checksums"))
+			files = FALSE;
+		if (g_str_has_prefix (changes_file[c], "Files"))
+			files = TRUE;
+		if (!files)
+			continue;
 		debline = NULL;
 		debname = NULL;
 		debline = g_strrstr (changes_file[c], ".deb");
 		if (debline)
 		{
+			GFile * debfile;
 			gchar * partial;
+
+			/* g_str is safe because this line comes from inside the
+			.changes file, not from a path or filename */
+			if (!g_str_has_suffix (debline, ".deb"))
+				continue;
+			debfile = NULL;
 			partial = g_strstrip (g_strrstr (changes_file[c], " "));
-			if (g_str_has_suffix (chgpath, "/"))
-				debname = g_strconcat (chgpath, partial, NULL);
-			else
-				debname = g_strconcat(chgpath, "/", partial, NULL);
+			debname = g_strconcat(chgpath, "/", partial, NULL);
+			debfile = g_file_new_for_uri (debname);
 			if (g_str_has_suffix (debname, ".deb"))
-				dv_new_window (debname);
+				dv_new_window (debfile);
 		}
 	}
 }
@@ -192,6 +229,8 @@ dv_archive_free (DVContents *deb)
 		g_hash_table_destroy (deb->data_files);
 	if (deb->copyright)
 		g_free (deb->copyright);
+	if (deb->ghandle)
+		g_object_unref (deb->ghandle);
 	g_free (deb);
 	preview_shutdown ();
 	deb = NULL;
@@ -232,6 +271,9 @@ void
 dv_set_working_path (DVContents * deb, gchar * path)
 {
 	g_return_if_fail (deb);
+	if (!path)
+		return;
+	g_free (deb->current_path);
 	deb->current_path = g_strdup (path);
 }
 
@@ -303,23 +345,28 @@ dv_show_error (DVError dve, GtkWidget * widget)
 }
 
 void
-set_deb_file (const gchar * filename, DVContents *deb)
+set_deb_file (GFile * gfile, DVContents *deb)
 {
+	gchar * filename;
 	g_return_if_fail (deb);
-	if (!filename)
+	if (!gfile)
 	{
 		dv_show_error (DV_ERR_NO_FILE, deb->parent);
 		return;
 	}
+	if (!deb->ghandle)
+		deb->ghandle = gfile;
+	filename = g_file_get_uri (deb->ghandle);
 	if ((!g_str_has_suffix(filename, ".deb")) &&
 		(!g_str_has_suffix(filename, ".udeb")))
 		return;
 	deb->filename = filename;
+	deb->ghandle = gfile;
 }
 
 static gint
 dv_archive_open (struct archive *a, gpointer client_data)
-{	
+{
 	DVContents *d = (DVContents*)client_data;
 	gchar *size;
 
@@ -330,8 +377,11 @@ dv_archive_open (struct archive *a, gpointer client_data)
 	size = g_strndup (d->current->ar_size, 10);
 	d->skip = g_ascii_strtod (size, NULL);
 	g_free (size);
-	d->readbuffer = malloc (d->skip);
-	if( d->readbuffer == NULL ) {
+	if (d->skip == 0)
+		return ARCHIVE_FATAL;
+	d->readbuffer = g_strnfill (d->skip, ' ');
+	if( d->readbuffer == NULL )
+	{
 		archive_set_error(a, ENOMEM, _("Out of memory."));
 		d->ar_message = archive_error_string (a);
 		return ARCHIVE_FATAL;
@@ -340,34 +390,35 @@ dv_archive_open (struct archive *a, gpointer client_data)
 }
 
 static ssize_t
-dv_archive_read (struct archive *a, gpointer client_data,
-	gconstpointer *buffer)
+dv_archive_read (struct archive *a, gpointer client_data, gconstpointer *buffer)
 {
-	GnomeVFSFileSize bytesread;
-	GnomeVFSResult result;
+	gsize bytesread;
+	GError * result;
 
 	DVContents *d = (DVContents*)client_data;
 	g_return_val_if_fail (d, DV_AR_FAIL);
+	result = NULL;
 	*buffer = d->readbuffer;
 	g_return_val_if_fail (d->readbuffer != NULL, DV_AR_FAIL);
 	if (d->skip == 0)
 		return ARCHIVE_OK;
-	result = gnome_vfs_read (d->vfshandle, d->readbuffer, d->skip, &bytesread);
-	if (result != GNOME_VFS_OK)
+	if (g_input_stream_is_closed (d->stream))
+		return DV_AR_FAIL;
+	bytesread = g_input_stream_read (d->stream, d->readbuffer, d->skip, NULL, &result);
+	if (result)
 	{
-		g_warning ("VFS read error:%s", gnome_vfs_result_to_string (result));
-		archive_set_error (a, errno, 
-			_("Error reading from file: %m"));
+		archive_set_error (a, errno, _("Error reading from file: %m"));
 		d->ar_message = archive_error_string (a);
 		dv_show_error (DV_ERR_FILE_READ, d->parent);
+		g_clear_error (&result);
 		return DV_AR_FAIL;
 	}
 	if (bytesread == 0)
 	{
-		archive_set_error (a, EIO, 
-			_("Unexpected end of file."));
+		archive_set_error (a, EIO, _("Unexpected end of file."));
 		d->ar_message = archive_error_string (a);
 		dv_show_error (DV_ERR_FILE_READ, d->parent);
+		g_clear_error (&result);
 		return DV_AR_FAIL;
 	}
 	d->skip -= bytesread;
@@ -375,7 +426,7 @@ dv_archive_read (struct archive *a, gpointer client_data,
 }
 
 /* A .deb is two archives within an archive so do NOT
-close the FILE or GnomeVFSHandle at this stage! */
+close the FILE at this stage! */
 static gint
 dv_archive_close (struct archive *r __attribute__ ((unused)), gpointer client_data)
 {
@@ -406,53 +457,76 @@ parseheaderlength (const gchar *inh, size_t len)
 	return r;
 }
 
-void
-prepare_contents (const gchar * filename, DVContents * deb)
+static gboolean
+prepare_contents (GFile * gfile, DVContents * deb)
 {
-	GnomeVFSFileSize bytesread, header;
-	GnomeVFSURI * vfs_uri;
-	GnomeVFSResult result;
-	GnomeVFSFileOffset seek;
+	gsize bytesread, header, seek;
+	GError * result;
+	GFileInputStream * filestream;
 	struct ar_hdr debian_binary;
 	struct ar_hdr controlgz;
 	struct ar_hdr datagz;
 	gint memberlen, error_num;
 	gulong bytes_left;
-	gchar * versionbuf, * vers_cmp, *infobuf;
+	gchar * versionbuf, * vers_cmp, *infobuf, * filename;
 
-	g_return_if_fail (deb);
-	vfs_uri = gnome_vfs_uri_new (gnome_vfs_make_uri_from_shell_arg (filename));
-	if (!gnome_vfs_uri_exists (vfs_uri))
+	g_return_val_if_fail (deb, FALSE);
+	result = NULL;
+	deb->ghandle = gfile;
+	filestream = g_file_read (deb->ghandle, NULL, &result);
+	if (result)
 	{
+		filename = g_file_get_parse_name (deb->ghandle);
+		g_warning ("Error: %s : %s", filename, result->message);
 		dv_show_error (DV_ERR_FILE_READ, deb->parent);
-		return;
-	}
-	result = gnome_vfs_open_uri (&deb->vfshandle, vfs_uri, GNOME_VFS_OPEN_READ);
-	if (result != GNOME_VFS_OK)
-	{
-		dv_show_error (DV_ERR_FILE_READ, deb->parent);
-		g_warning ("VFS open error: %s", gnome_vfs_result_to_string (result));
-		gnome_vfs_close (deb->vfshandle);
-		return;
+		g_free (filename);
+		g_clear_error (&result);
+		return FALSE;
 	}
+	deb->stream = (GInputStream*)filestream;
+	if (g_input_stream_is_closed (deb->stream))
+		return FALSE;
 	versionbuf = g_new0 (gchar, strlen(ARMAG) + 1);
-	gnome_vfs_read (deb->vfshandle, versionbuf, strlen(ARMAG), &bytesread);
+	bytesread = g_input_stream_read (deb->stream, versionbuf, strlen(ARMAG), NULL, &result);
+	if (result)
+	{
+		g_warning (result->message);
+		g_clear_error (&result);
+		return FALSE;
+	}
 	vers_cmp = g_strndup (versionbuf, strlen(ARMAG));
 	g_free (versionbuf);
 	if (strcmp(vers_cmp, ARMAG))
 	{
 		dv_show_error (DV_ERR_VERSION, deb->parent);
-		gnome_vfs_close (deb->vfshandle);
-		return;
+		g_input_stream_close (deb->stream, NULL, &result);
+		if (result)
+		{
+			g_warning (result->message);
+			g_clear_error (&result);
+		}
+		return FALSE;
 	}
 	header = sizeof(struct ar_hdr);
-	gnome_vfs_read (deb->vfshandle, &debian_binary, header, &bytesread);
+	bytesread = g_input_stream_read (deb->stream, &debian_binary, header, NULL, &result);
 	if (memcmp (debian_binary.ar_name, "debian-binary   ",
 			sizeof (debian_binary.ar_name)) &&
 		memcmp (debian_binary.ar_name, "debian-binary/   ",
 			sizeof (debian_binary.ar_name)))
 	{
 		dv_show_error (DV_ERR_NOT_DPKG, deb->parent);
+		g_input_stream_close (deb->stream, NULL, &result);
+		if (result)
+		{
+			g_warning (result->message);
+			g_clear_error (&result);
+		}
+		return FALSE;
+	}
+	if (result)
+	{
+		g_warning (result->message);
+		g_clear_error (&result);
 	}
 	memberlen = parseheaderlength (debian_binary.ar_size, 
 		sizeof (debian_binary.ar_size));
@@ -460,10 +534,20 @@ prepare_contents (const gchar * filename, DVContents * deb)
 		dv_show_error (DV_ERR_CORRUPT_FILE, deb->parent);
 	header = memberlen + 1;
 	infobuf = g_new0(gchar, header);
-	gnome_vfs_read (deb->vfshandle, infobuf, memberlen + (memberlen & 1), &bytesread);
+	bytesread = g_input_stream_read (deb->stream, infobuf, memberlen + (memberlen & 1), NULL, &result);
+	if (result)
+	{
+		g_warning (result->message);
+		g_clear_error (&result);
+	}
 	g_free (infobuf);
 	header = sizeof(struct ar_hdr);
-	gnome_vfs_read (deb->vfshandle, &controlgz, header, &bytesread);
+	bytesread = g_input_stream_read (deb->stream, &controlgz, sizeof (struct ar_hdr), NULL, &result);
+	if (result)
+	{
+		g_warning (result->message);
+		g_clear_error (&result);
+	}
 	deb->current = &controlgz;
 	deb->control_tar = archive_read_new ();
 	archive_read_support_compression_gzip (deb->control_tar);
@@ -475,8 +559,13 @@ prepare_contents (const gchar * filename, DVContents * deb)
 	{
 		dv_show_error (DV_ERR_CORRUPT_FILE, deb->parent);
 		deb->ar_message = archive_error_string (deb->control_tar);
-		gnome_vfs_close (deb->vfshandle);
-		return;
+		g_input_stream_close (deb->stream, NULL, &result);
+		if (result)
+		{
+			g_warning (result->message);
+			g_clear_error (&result);
+		}
+		return FALSE;
 	}
 	else
 	{
@@ -521,23 +610,23 @@ prepare_contents (const gchar * filename, DVContents * deb)
 	seek = (bytes_left & 1);
 	if (seek > 0)
 	{
-		gpointer dump;
-		dump = g_new0(gchar, seek);
-		/* seek doesn't appear to work well in VFS so read into a dump instead. */
-		result = gnome_vfs_read (deb->vfshandle, dump, seek, &bytesread);
-		g_free (dump);
-		if (result != GNOME_VFS_OK) {
+		g_input_stream_skip (deb->stream, seek, NULL, &result);
+		if (result) {
 			dv_show_error (DV_ERR_CORRUPT_FILE, deb->parent);
 			deb->ar_message = archive_error_string (deb->control_tar);
-			gnome_vfs_close (deb->vfshandle);
-			return;
+			g_input_stream_close (deb->stream, NULL, &result);
+			g_warning (result->message);
+			g_clear_error (&result);
+			return FALSE;
 		}
 	}
 	header = sizeof(struct ar_hdr);
-	result = gnome_vfs_read (deb->vfshandle, &datagz, header, &bytesread);
-	if (result != GNOME_VFS_OK)
-		g_warning ("VFS data tarball read error: %s", 
-		gnome_vfs_result_to_string (result));
+	bytesread = g_input_stream_read (deb->stream, &datagz, sizeof(struct ar_hdr), NULL, &result);
+	if (result)
+	{
+		g_warning ("Data tarball read error: %s", result->message);
+		g_clear_error (&result);
+	}
 	deb->current = &datagz;
 	deb->data_tar = archive_read_new ();
 	archive_read_support_compression_gzip (deb->data_tar);
@@ -549,6 +638,12 @@ prepare_contents (const gchar * filename, DVContents * deb)
 	{
 		dv_show_error (DV_ERR_CORRUPT_FILE, deb->parent);
 		deb->ar_message = archive_error_string (deb->data_tar);
+		g_input_stream_close (deb->stream, NULL, &result);
+		if (result)
+		{
+			g_warning (result->message);
+			g_clear_error (&result);
+		}
 	}
 	else
 	{
@@ -602,7 +697,13 @@ prepare_contents (const gchar * filename, DVContents * deb)
 		}
 		archive_read_finish (deb->data_tar);
 	}
-	gnome_vfs_close (deb->vfshandle);
+	g_input_stream_close (deb->stream, NULL, &result);
+	if (result)
+	{
+		g_warning (result->message);
+		g_clear_error (&result);
+	}
+	return TRUE;
 }
 
 static void
@@ -722,8 +823,9 @@ dv_update_title (const gchar * filename, DVContents * deb)
 	gtk_window_set_title (GTK_WINDOW(deb->parent), newstr);
 }
 
+/* receives new deb from toolbar selection. */
 void
-open_deb (const gchar * filename, DVContents * deb)
+open_deb (DVContents * deb)
 {
 	GtkWidget *treeview;
 	GtkTreeStore *store;
@@ -735,21 +837,31 @@ open_deb (const gchar * filename, DVContents * deb)
 	GtkTreeViewColumn *column;
 	GtkTreeSelection *select;
 	DVIter dvi;
-	GnomeVFSFileInfo *vfsinfo;
-	GnomeVFSResult vfsresult;
+	GFileInfo * ginfo;
+	GError * result;
+	gchar * filename;
 
+	result = NULL;
 	g_return_if_fail (deb);
-	if (!filename)
-		return;
-	vfsinfo = gnome_vfs_file_info_new ();
-	vfsresult = gnome_vfs_get_file_info (filename, vfsinfo, 
-		GNOME_VFS_FILE_INFO_DEFAULT);
-	if ((vfsinfo->type != GNOME_VFS_FILE_TYPE_REGULAR) || (vfsinfo->size == 0))
+	filename = g_file_get_parse_name (deb->ghandle);
+	if (g_file_query_exists (deb->ghandle, NULL))
 	{
-		dv_show_error (DV_ERR_NO_FILE, deb->parent);
-		return;
+		ginfo = g_file_query_info (deb->ghandle, G_FILE_ATTRIBUTE_STANDARD_SIZE,
+			G_FILE_QUERY_INFO_NONE, NULL, &result);
+		if (result)
+		{
+			g_warning (result->message);
+			g_clear_error (&result);
+		}
+		if (0 == g_file_info_get_attribute_uint64 (ginfo,
+			G_FILE_ATTRIBUTE_STANDARD_SIZE))
+		{
+			dv_show_error (DV_ERR_NO_FILE, deb->parent);
+			return;
+		}
 	}
-	prepare_contents (filename, deb);
+	if (!prepare_contents (deb->ghandle, deb))
+		return;
 	dv_update_title(filename, deb);
 	treeview = lookup_widget (deb->parent, "treeview");
 	g_return_if_fail (treeview);
@@ -819,7 +931,11 @@ dv_archive_preload (GtkWidget * widget)
 
 	deb = (DVContents*) lookup_widget (widget, DVCONTENTS);
 	if (deb)
-		open_deb (deb->filename, deb);
+	{
+		if (!deb->ghandle)
+			return;
+		open_deb (deb);
+	}
 }
 
 gchar *
