@@ -141,7 +141,7 @@ create_pixbuf (const gchar *filename)
     {
       fprintf (stderr, "Failed to load pixbuf file: %s: %s\n",
                pathname, error->message);
-      g_error_free (error);
+      g_clear_error (&error);
     }
   g_free (pathname);
   return pixbuf;
